"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonSerializer = void 0;
/**
 * A naive and generally unsafe default JSON serializer. This relies on
 * the native JSON.stringify/parse functions that do a basic job of serialization.
 * Deserialized forms will be plain objects with class properties (eg Date) not
 * being a strong type.
 *
 * It's recommended that an external serializer like `@node-ts/bus-class-serializer`
 * be used instead that will preserve types when serialized/deserialized.
 */
class JsonSerializer {
    serialize(obj) {
        return JSON.stringify(obj);
    }
    deserialize(serialized, classConstructor) {
        const plain = JSON.parse(serialized);
        return this.toClass(plain, classConstructor);
    }
    toPlain(obj) {
        return JSON.parse(JSON.stringify(obj));
    }
    toClass(obj, classConstructor) {
        const instance = new classConstructor();
        Object.assign(instance, obj);
        return instance;
    }
}
exports.JsonSerializer = JsonSerializer;
