"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageSerializer = void 0;
/**
 * This a wrapper around the real serializer.
 * Unlike JsonSerializer, whose sole job is parsing data,
 * this class will do some plumbing work to look up the Handler Registry for
 * the message constructor.
 *
 * Normally, transports will use this instead of the real serializer.
 */
class MessageSerializer {
    constructor(serializer, handlerRegistry) {
        this.serializer = serializer;
        this.handlerRegistry = handlerRegistry;
    }
    serialize(message) {
        return this.serializer.serialize(message);
    }
    deserialize(serializedMessage) {
        const naiveDeserializedMessage = JSON.parse(serializedMessage);
        const messageType = this.handlerRegistry.getMessageConstructor(naiveDeserializedMessage.$name);
        return (!!messageType
            ? this.serializer.deserialize(serializedMessage, messageType)
            : naiveDeserializedMessage);
    }
}
exports.MessageSerializer = MessageSerializer;
