"use strict";
/**
 * This middleware pattern has been grabbed from here:
 * https://evertpot.com/generic-middleware/
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.MiddlewareDispatcher = void 0;
const tslib_1 = require("tslib");
/**
 * A middleware container and invoker
 */
class MiddlewareDispatcher {
    constructor(finalMiddlewares = []) {
        this.finalMiddlewares = finalMiddlewares;
        this.middlewares = [];
    }
    /**
     * Add a middleware function.
     */
    use(...middlewares) {
        this.middlewares.push(...middlewares);
    }
    /**
     * Add 'final' middlewares that will be added to the end of the
     * regular middlewares. This allows for finer control when exposing
     * the @see use functionality to consumers but wanting to ensure that your
     * final middleware is last to run
     */
    useFinal(...middlewares) {
        this.finalMiddlewares.push(...middlewares);
    }
    /**
     * Execute the chain of middlewares, in the order they were added on a
     * given Context.
     */
    dispatch(context) {
        return invokeMiddlewares(context, this.middlewares.concat(this.finalMiddlewares));
    }
}
exports.MiddlewareDispatcher = MiddlewareDispatcher;
function invokeMiddlewares(context, middlewares) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (!middlewares.length) {
            return;
        }
        const middleware = middlewares[0];
        return middleware(context, () => tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield invokeMiddlewares(context, middlewares.slice(1));
        }));
    });
}
