"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TypedEmitter = void 0;
/**
 * An EventEmitter that emits a strongly typed event
 * @see https://basarat.gitbook.io/typescript/main-1/typed-event
 */
class TypedEmitter {
    constructor() {
        this.listeners = [];
        this.listenersOncer = [];
        this.on = (listener) => {
            this.listeners.push(listener);
            return () => this.off(listener);
        };
        this.once = (listener) => {
            this.listenersOncer.push(listener);
        };
        this.off = (listener) => {
            var callbackIndex = this.listeners.indexOf(listener);
            if (callbackIndex > -1)
                this.listeners.splice(callbackIndex, 1);
        };
        this.emit = (event) => {
            // Update any general listeners
            this.listeners.forEach(listener => listener(event));
            // Clear the `once` queue
            if (this.listenersOncer.length > 0) {
                const toCall = this.listenersOncer;
                this.listenersOncer = [];
                toCall.forEach(listener => listener(event));
            }
        };
        this.pipe = (typedEmitter) => {
            return this.on(e => typedEmitter.emit(e));
        };
    }
}
exports.TypedEmitter = TypedEmitter;
