import { Persistence } from './persistence';
import { WorkflowState } from '../workflow-state';
import { MessageWorkflowMapping } from '../message-workflow-mapping';
import { Message, MessageAttributes } from '@node-ts/bus-messages';
import { ClassConstructor, CoreDependencies } from '../../util';
/**
 * A non-durable in-memory persistence for storage and retrieval of workflow state. Before using this,
 * be warned that all workflow state will not survive a process restart or application shut down. As
 * such this should only be used for testing, prototyping or handling unimportant workflows.
 */
export declare class InMemoryPersistence implements Persistence {
    private workflowState;
    private logger;
    prepare(coreDependencies: CoreDependencies): void;
    initializeWorkflow<TWorkflowState extends WorkflowState>(workflowStateConstructor: ClassConstructor<TWorkflowState>, _: MessageWorkflowMapping<Message, WorkflowState>[]): Promise<void>;
    getWorkflowState<WorkflowStateType extends WorkflowState, MessageType extends Message>(workflowStateConstructor: ClassConstructor<WorkflowStateType>, messageMap: MessageWorkflowMapping<MessageType, WorkflowStateType>, message: MessageType, attributes: MessageAttributes, includeCompleted?: boolean | undefined): Promise<WorkflowStateType[]>;
    saveWorkflowState<WorkflowStateType extends WorkflowState>(workflowState: WorkflowStateType): Promise<void>;
    length(workflowStateConstructor: ClassConstructor<WorkflowState>): number;
}
