import { Message, MessageAttributes } from '@node-ts/bus-messages';
import { ClassConstructor } from '../util';
import { MessageWorkflowMapping } from './message-workflow-mapping';
import { WorkflowState, WorkflowStatus } from './workflow-state';
export type WorkflowHandler<TMessage extends Message, TMessageAttributes extends MessageAttributes, WorkflowStateType extends WorkflowState> = (message?: TMessage, attributes?: TMessageAttributes, workflowState?: WorkflowStateType) => void | Partial<WorkflowStateType> | Promise<void | Partial<WorkflowStateType>>;
export type WhenHandler<WorkflowStateType extends WorkflowState, WorkflowType extends Workflow<WorkflowStateType>> = (workflow: WorkflowType) => WorkflowHandler<Message, MessageAttributes, WorkflowStateType>;
type KeyOfType<T, U> = {
    [P in keyof T]: T[P] extends U ? P : never;
}[keyof T];
export type OnWhenHandler<WorkflowStateType extends WorkflowState = WorkflowState, WorkflowType extends Workflow<WorkflowStateType> = Workflow<WorkflowStateType>> = {
    workflowCtor: ClassConstructor<Workflow<WorkflowState>>;
    workflowHandler: KeyOfType<WorkflowType, Function>;
    customLookup: MessageWorkflowMapping | undefined;
};
/**
 * A workflow configuration that describes how to map incoming messages to handlers within the workflow.
 */
export declare class WorkflowMapper<WorkflowStateType extends WorkflowState, WorkflowType extends Workflow<WorkflowStateType>> {
    private readonly workflow;
    readonly onStartedBy: Map<ClassConstructor<Message>, {
        workflowCtor: ClassConstructor<Workflow<WorkflowState>>;
        workflowHandler: KeyOfType<WorkflowType, Function>;
    }>;
    readonly onWhen: Map<ClassConstructor<Message>, OnWhenHandler<WorkflowStateType, WorkflowType>>;
    private workflowStateType;
    constructor(workflow: ClassConstructor<Workflow<WorkflowState>>);
    get workflowStateCtor(): ClassConstructor<WorkflowStateType> | undefined;
    withState(workflowStateType: ClassConstructor<WorkflowStateType>): this;
    startedBy<MessageType extends Message>(message: ClassConstructor<MessageType>, workflowHandler: KeyOfType<WorkflowType, Function>): this;
    when<MessageType extends Message>(message: ClassConstructor<MessageType>, workflowHandler: KeyOfType<WorkflowType, Function>, customLookup?: MessageWorkflowMapping<MessageType, WorkflowStateType>): this;
}
export declare abstract class Workflow<WorkflowStateType extends WorkflowState> {
    abstract configureWorkflow(mapper: WorkflowMapper<WorkflowStateType, any>): void;
    /**
     * Ends the workflow and optionally sets any final state. After this is returned,
     * the workflow instance will no longer be activated for subsequent messages.
     */
    protected completeWorkflow(workflowState?: Partial<WorkflowStateType>): {
        $status: WorkflowStatus;
        $workflowId?: WorkflowStateType["$workflowId"] | undefined;
        $version?: WorkflowStateType["$version"] | undefined;
        $name?: WorkflowStateType["$name"] | undefined;
    };
    /**
     * Prevents a new workflow from starting, and prevents the persistence of
     * the workflow state. This should only be used in `startedBy` workflow handlers.
     */
    protected discardWorkflow(): {
        $status: WorkflowStatus;
    };
}
export {};
