"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Workflow = exports.WorkflowMapper = void 0;
const error_1 = require("./error");
const workflow_state_1 = require("./workflow-state");
/**
 * A workflow configuration that describes how to map incoming messages to handlers within the workflow.
 */
class WorkflowMapper {
    constructor(workflow) {
        this.workflow = workflow;
        this.onStartedBy = new Map();
        this.onWhen = new Map();
    }
    get workflowStateCtor() {
        return this.workflowStateType;
    }
    withState(workflowStateType) {
        this.workflowStateType = workflowStateType;
        return this;
    }
    startedBy(message, workflowHandler
    // workflowHandler: (workflow: WorkflowType) => WorkflowHandler<MessageType, WorkflowStateType>
    ) {
        if (this.onStartedBy.has(message)) {
            throw new error_1.WorkflowAlreadyStartedByMessage(this.workflow.name, message);
        }
        this.onStartedBy.set(message, {
            workflowHandler,
            workflowCtor: this.workflow
        });
        return this;
    }
    when(message, workflowHandler, customLookup) {
        if (this.onWhen.has(message)) {
            throw new error_1.WorkflowAlreadyHandlesMessage(this.workflow.name, message);
        }
        this.onWhen.set(message, {
            workflowHandler,
            workflowCtor: this.workflow,
            customLookup: customLookup
        });
        return this;
    }
}
exports.WorkflowMapper = WorkflowMapper;
class Workflow {
    /**
     * Ends the workflow and optionally sets any final state. After this is returned,
     * the workflow instance will no longer be activated for subsequent messages.
     */
    completeWorkflow(workflowState) {
        return Object.assign(Object.assign({}, workflowState), { $status: workflow_state_1.WorkflowStatus.Complete });
    }
    /**
     * Prevents a new workflow from starting, and prevents the persistence of
     * the workflow state. This should only be used in `startedBy` workflow handlers.
     */
    discardWorkflow() {
        return { $status: workflow_state_1.WorkflowStatus.Discard };
    }
}
exports.Workflow = Workflow;
